# # -*- coding: utf-8 -*-
# """
# AAMS client module.
# Comunica direttamente verso il concessionario o AAMS.
#
# NB: NON COMPLETATO E NON UTILIZZABILE
# """
#
# import pytz
# import logging
#
# from datetime import datetime
# from typing import Optional
#
# from requests import Response
#
# from thx.giocaonline.client.base_aams_client_mixin import AAMSClientMixin
# from thx.giocaonline.constants import SessionAttributes, Region, FlagRiacquisto, CodeRete, CodeTipoTag
# from thx.giocaonline.exceptions import ValidationError
#
# logger = logging.getLogger('giocaonline')
#
#
# class AAMSClient(AAMSClientMixin):
#     """AAMS HTTP client
#
#     SUPPORTO:
#
#     - le modalità di gestione dei flussi di comunicazione ai fini dello svolgimento del gioco IMPLEMENTATE
#       sono le seguenti: 1) e 2). Le modalità 3 e 4 non sono supportate.
#
#     - Tipologie di gioco supportate: Giochi di abilità.
#
#     1. Messaggi per lo svolgimento del gioco:
#         A. Giochi di abilità (di cui all’art. 1 comma1 del su indicato decreto):
#             1. Inizio sessione di gioco (200) (§ 6.3.1)
#             2. Acquisto diritto di partecipazione (220) (§ 6.3.2)
#             3. Richiesta di annullamento diritto di partecipazione (230) (§ 6.3.3)
#             4. Piano dei premi (240) (§ 6.3.4)
#             5. Convalida della sessione (250) (§ 6.3.5)
#             6. Lista vincitori (260) (§6.3.6)
#             7. Accredito vincita (280) (§ 6.3.7)
#             8. Fine sessione di gioco(300) (§ 6.3.8)
#
#     2. Messaggi per la gestione degli archivi:
#         a. Comuni a tutte le tipologie di gioco/modalità di flusso
#             1. Richiesta dati di rendicontazione contabile (800) (§ 6.9.1)
#             2. Richiesta di aggiornamento data fine sessione (810) (§ 6.9.2)
#             3. Comunicazione del software installato (830) (§ 6.9.3)
#             4. Comunicazione del software installato per versione (831) (§ 6.9.3)
#             5. Aggiunta di certificato di firma (840) (§ 6.9.3)
#         b. Distinti per tipologia di gioco/modalità di flusso
#             A. Giochi di abilità (di cui all’art. 1 comma1 del su indicato decreto):
#                 1. Richiesta invalidazione sessione (310) (§ 6.6.1)
#                 2. Richiesta invalidazione diritto di partecipazione (320) (§ 6.6.2)
#                 3. Verifica stato della richiesta di invalidazione (330) (§ 6.6.3)
#                 4. Accredito rimborso (340) (§ 6.6.4)
#                 5. Verifica correttezza sessioni effettuate (360) (§ 6.6.5)
#                 6. Richiesta notifica anomalie riscontrate per una sessione (365) (§ 6.6.6)
#     """
#     current_message: Optional[int] = None
#     data: Optional[dict] = None
#
#     _methods_map: dict = {
#         200: 'inizio_sessione_200',
#         220: 'acquisto_diritto_di_partecipazione_220',
#         230: 'richiesta_annullamento_diritto_di_partecipazione_230',
#         240: 'piano_dei_premi_240',
#         250: 'convalida_della_sessione_250',
#         260: 'lista_vincitori_260',
#         280: 'accredito_vincita_280',
#
#         300: 'fine_sessione_300',
#         310: 'richiesta_invalidazione_sessione_310',
#         320: 'richiesta_invalidazione_diritto_partecipazione_320',
#         330: 'verifica_stato_richiesta_invalidazione_330',
#         340: 'accredito_rimborso_340',
#         360: 'verifica_correttezza_sessioni_effettuate_360',
#         365: 'richiesta_notifica_anomalie_riscontrate_per_sessione_365',
#
#         800: 'richiesta_dati_rendicondazione_contabile_800',
#         810: 'richiesta_aggiornamento_data_fine_sessione_810',
#         830: 'comunicazione_del_software_installato_830',
#         831: 'comunicazione_del_software_installato_per_versione_831',
#         840: 'aggiunta_del_certificato_di_firma_840',
#     }
#
#     endpoints: dict = {
#         200: '',
#         220: '',
#         230: '',
#         240: '',
#         250: '',
#         260: '',
#         280: '',
#
#         300: '',
#         310: '',
#         320: '',
#         330: '',
#         340: '',
#         360: '',
#         365: '',
#
#         800: '',
#         810: '',
#         830: '',
#         831: '',
#         840: '',
#     }
#
#     # MESSAGGI PER LO SVOLGIMENTO DEL GIOCO IN MODALITA’ 1 E 2. A partire da pagina 30 della documentazione.
#
#     # 200
#     def inizio_sessione_200(
#         self,
#         session_id: str,
#         session_datetime_start: datetime,
#         session_datetime_end: datetime,
#         communication_modality: str,
#         imp_diritto_partecip: float,  # _type: €Int
#         **session_attributes
#     ):
#         """
#         Questo messaggio consente ad un concessionario di comunicare l’avvio di una sessione di gioco.
#
#         :param session_id: Identificativo univoco della sessione di gioco attribuito dal concessionario proponente
#             (len: 16)
#         :param session_datetime_start: Data inizio sessione di gioco (UTC)
#         :param session_datetime_end: Data di fine sessione di gioco (UTC).
#             Deve essere impostata alla data fine della sessione di gioco “presunta”
#             (si intende valida fino alle ore 24 del giorno indicato); qualora il
#             concessionario si accorga, durante lo svolgimento del gioco, di non riuscire ad effettuare
#             la chiusura entro la data dichiarata, dovrà inoltrare il messaggio di proroga (messaggio
#             810), al fine di poter completare correttamente la sessione di gioco; I limiti della durata
#             di una sessione di gioco sono determinati da ADM.
#         :param communication_modality: Flusso di gestione delle comunicazioni prescelto dal concessionario (vale 1 o 2)
#         :param imp_diritto_partecip: Contiene il valore dell’importo del diritto di partecipazione il
#             cui valore deve rispettare i limiti definiti dalla normativa.
#         """
#         return self._do_request_message(
#             message_code=200, signed=True,
#             session_id=session_id,
#             session_datetime_start=session_datetime_start,
#             session_datetime_end=session_datetime_end,
#             communication_modality=communication_modality,
#             imp_diritto_partecip=imp_diritto_partecip,
#             **session_attributes
#         )
#
#     def validate_and_prepare_data_200(
#         self,
#         session_id: str,
#         session_datetime_start: datetime,
#         session_datetime_end: datetime,
#         communication_modality: str,
#         imp_diritto_partecip: int,
#         **session_attributes
#     ):
#         self.check_length('session_id', session_id, 16, "e")
#         self.check_timezone('session_datetime_start', session_datetime_start, pytz.utc)
#         self.check_timezone('session_datetime_end', session_datetime_end, pytz.utc)
#         self.check_numeric_value('imp_diritto_partecip', imp_diritto_partecip, 0, "ge")
#         self.check_value_in('communication_modality', communication_modality, (
#             "1",  # non condizionata
#             "2",  # condizionata
#         ))
#
#         attributes, attributes_count = self._validate_and_prepare_session_attributes(session_attributes)
#
#         # PREPARE DATA
#         self.data = {
#             # session_id
#             "campo1": session_id,
#             # session_datetime_start
#             "campo2": session_datetime_start.day,
#             "campo3": session_datetime_start.month,
#             "campo4": session_datetime_start.year,
#             "campo5": session_datetime_start.hour,
#             "campo6": session_datetime_start.minute,
#             "campo7": session_datetime_start.second,
#             # session_datetime_end
#             "campo8": session_datetime_end.day,
#             "campo9": session_datetime_end.month,
#             "campo10": session_datetime_end.year,
#             # Attributi caratteristici della sessione di gioco
#             "campo11": communication_modality,
#             "campo12": imp_diritto_partecip,
#             "campo13": attributes_count,
#             "campo14": attributes
#         }
#
#     @staticmethod
#     def _validate_and_prepare_session_attributes(session_attributes: dict) -> tuple:
#         attributes = {}
#         cnt = 0
#         for k, v in session_attributes.items():
#             v = SessionAttributes.validate(k, v)
#             cnt += 1
#             attributes[f"codice{cnt}"] = k
#             attributes[f"valore{cnt}"] = str(v)
#
#         return attributes, cnt
#
#     # 220
#     def acquisto_diritto_di_partecipazione_220(
#         self,
#         id_sess_sist_centr: str,
#         cod_regione: int,
#         imp_partecipazione: float,  # _type: €Int
#         imp_partecipazione_bonus: float,  # _type: €Int
#         imp_a_fondo_jackpot: float,  # _type: €Int
#         flag_riacquisto: int,
#         ip_address: str,
#         code_concessionario: int,
#         code_rete: int,
#         code_conto_gioco: str,
#         pseudonimo_giocatore: str,
#         data: datetime,
#         id_sezione_in: str,
#         code_tipo_tag: int,
#         id_sessione_collegata: Optional[str] = None,
#     ) -> tuple:
#         """
#         Questo messaggio consente ad un concessionario di inoltrare al sistema centrale la
#         richiesta di acquisto del “biglietto elettronico” necessario ad un giocatore per
#         partecipare al gioco.
#
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param id_sessione_collegata: Codice identificativo della sessione attribuito in precedenza dal sistema
#              centrale alla sessione collegata. Deve essere valorizzato qualora per l’acquisto del
#              diritto di partecipazione venga utilizzato l’importo della vincita ottenuta dal giocatore
#              nella sessione indicata precedentemente svoltasi.
#         :param cod_regione: Impostare con il codice regione (vedi tabella § 8.4) di residenza del titolare del conto
#              di gioco.
#         :param imp_partecipazione: Importo del diritto di partecipazione che permette l’ingresso al gioco.
#              assume il valore dell’importo del diritto di partecipazione (o di eventuali add-on o rebuy per sessioni
#              che lo prevedono).
#         :param imp_partecipazione_bonus: Quota dell’importo di partecipazione derivante da bonus.
#              deve essere impostato al valore dell’importo del diritto di partecipazione derivante da un BONUS;
#              vale se all’apertura della sessione di gioco sia stata comunicata tale possibilità.
#         :param flag_riacquisto: Impostare a 1 per un riacquisto, 2 per un add-on, 0 negli altri casi.
#              Nel caso in cui per la sessione di gioco in corso sia ammesso il riacquisto (rebuy o add-on)
#              di un diritto di partecipazione il campo 5 (flag riacquisto) conterrà, se previsto, il
#              valore 1 (o 2 per add-on) per gli eventuali acquisti successivi al primo, sempre nel
#              rispetto dei limiti imposti dalla normativa.
#         :param ip_address: Indirizzo IP del computer dal quale si connette il giocatore (comprensivo dei
#              punti; es. 127.0.0.1)
#         :param code_concessionario: Codice identificativo, assegnato da AAMS, del concessionario presso
#              cui è aperto il conto di gioco
#         :param code_rete: Codice identificativo della rete del concessionario titolare di sistema presso
#             c ui è aperto il conto di gioco (vedi tabella § 8.6)
#         :param code_conto_gioco: Da impostare con il codice che identifica il conto di gioco
#         :param pseudonimo_giocatore: Pseudonimo del giocatore associato al conto di gioco
#         :param data: Data acquisto diritto di partecipazione (UTC)
#         :param imp_a_fondo_jackpot: Importo destinato al fondo Jackpot
#         :param id_sezione_in: Lettera identificativa della sezione di primo livello di un torneo multifase
#              (lettera identificativa della sezione di ingresso). Da indicare per i tornei multifase,
#              valorizzandolo con una lettera progressiva tra A e la lettera corrispondente al valore massimo indicati
#              per l’attributo NSI del messaggio 200, cioè da A a massimo P. Indicare 0 se il torneo non è multifase.
#         :param code_tipo_tag: Impostare con il codice del tipo di terminale di gioco utilizzato dal giocatore
#              per partecipare alla sessione (vedi tabella codici tipo TAG)
#         """
#         return self._do_request_message(
#             message_code=220, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             id_sessione_collegata=id_sessione_collegata,
#             cod_regione=cod_regione,
#             imp_partecipazione=imp_partecipazione,
#             imp_partecipazione_bonus=imp_partecipazione_bonus,
#             flag_riacquisto=flag_riacquisto,
#             ip_address=ip_address,
#             code_concessionario=code_concessionario,
#             code_rete=code_rete,
#             code_conto_gioco=code_conto_gioco,
#             pseudonimo_giocatore=pseudonimo_giocatore,
#             data=data,
#             imp_a_fondo_jackpot=imp_a_fondo_jackpot,
#             id_sezione_in=id_sezione_in,
#             code_tipo_tag=code_tipo_tag,
#         )
#
#     def validate_and_prepare_data_220(
#         self,
#         id_sess_sist_centr: str,
#         cod_regione: int,
#         imp_partecipazione: float,  # _type: €Int
#         imp_partecipazione_bonus: float,  # _type: €Int
#         imp_a_fondo_jackpot: float,  # _type: €Int
#         flag_riacquisto: int,
#         ip_address: str,
#         code_concessionario: int,
#         code_rete: int,
#         code_conto_gioco: str,
#         pseudonimo_giocatore: str,
#         data: datetime,
#         id_sezione_in: str,
#         code_tipo_tag: int,
#         id_sessione_collegata: Optional[str] = None,
#     ):
#         # VALIDATION
#         id_sezione_in = id_sezione_in.upper() if isinstance(id_sezione_in, str) else id_sezione_in
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         if id_sessione_collegata is not None:
#             self.check_length('id_sessione_collegata', id_sessione_collegata, 16, "e")
#         self.check_length('code_conto_gioco', code_conto_gioco, 21, "l")
#         self.check_length('pseudonimo_giocatore', pseudonimo_giocatore, 101, "l")
#         self.check_length('ip_address', ip_address, 16, "l")
#         self.check_length('id_sezione_in', id_sezione_in, 1, "e")
#         self.check_value_in('id_sezione_in', id_sezione_in, "0ABCDEFGHILMNOP")
#         self.check_timezone('data', data, pytz.utc)
#         self.check_numeric_value('imp_partecipazione', imp_partecipazione, 0, "ge")
#         self.check_numeric_value('imp_partecipazione_bonus', imp_partecipazione_bonus, 0, "ge")
#         self.check_numeric_value('imp_a_fondo_jackpot', imp_a_fondo_jackpot, 0, "ge")
#         self.check_enum('cod_regione', str(cod_regione).rjust(2, "0"), Region)
#         self.check_enum('flag_riacquisto', flag_riacquisto, FlagRiacquisto)
#         self.check_enum('code_rete', code_rete, CodeRete)
#         self.check_enum('code_tipo_tag', CodeTipoTag.get(str(code_tipo_tag)), CodeTipoTag)
#
#         # PREPARE DATA
#         self.data = {
#             # session and player data
#             "Id_sess_sist_centr": id_sess_sist_centr,  # 1
#             "Cod_regione": cod_regione, # 2
#             "Imp_partecipazione": imp_partecipazione,  # 3
#             "Imp_partecipazione_bonus": imp_partecipazione_bonus, # 4
#             "Flag_riacquisto": flag_riacquisto, # 5
#             "Indirizzo_IP": ip_address, # 6
#             "Cod_conc_titolare": code_concessionario, # 7
#             "Cod_rete": code_rete,  # 8
#             "Lun_conto_gioco": len(code_conto_gioco),  # 9
#             "Cod_conto_gioco": code_conto_gioco,  # 10
#             "Lun_pseudonimo_giocatore": len(pseudonimo_giocatore),  # 11
#             "Pseudonimo_giocatore": pseudonimo_giocatore,  # 12
#             "Id_sessione_collegata": id_sessione_collegata,  # 13
#
#             # Data acquisto diritto di partecipazione
#             "Giorno": data.day,  # 15
#             "Mese": data.month,  # 16
#             "Anno": data.year,  # 17
#             "Ora": data.hour,  # 18
#             "Minuti": data.minute,  # 19
#             "Secondi": data.second,  # 20
#
#             "Imp_a_fondo_Jackpot": imp_a_fondo_jackpot,  # 21
#             "id_sezione_in": id_sezione_in,  # 22
#             "Cod_tipo_tag": code_tipo_tag,  # 23
#         }
#
#     # 230
#     def richiesta_annullamento_diritto_di_partecipazione_230(
#         self,
#         id_sess_sist_centr: str,
#         id_partecipazione: str,
#     ):
#         """
#         Tramite questo messaggio, il concessionario può richiedere l’annullamento di un diritto
#         di partecipazione acquistato in precedenza, solo per sessione di gioco in modalità 2 e
#         prima che sia stata convalidata (cfr paragrafo 5.4.2).
#         In altri casi l’annullamento può avvenire per motivi esclusivamente tecnici e disciplinati
#         da AAMS in appositi provvedimenti amministrativi, e tramite il flusso appositamente
#         predisposto (cfr paragrafo 5.5).
#
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param id_partecipazione: Codice univoco del diritto di partecipazione attribuito dal sistema centrale
#
#         """
#         return self._do_request_message(
#             message_code=230, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             id_partecipazione=id_partecipazione,
#         )
#
#     def validate_and_prepare_data_230(
#         self,
#         id_sess_sist_centr: str,
#         id_partecipazione: str
#     ):
#         # VALIDATION
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         self.check_length('id_partecipazione', id_partecipazione, 16, "e")
#
#         # PREPARE DATA
#         self.data = {
#             "campo1": id_sess_sist_centr,
#             "campo2": id_partecipazione,
#         }
#
#     # 240
#     def piano_dei_premi_240(
#         self,
#         id_sess_sist_centr: str,
#         perc_montepremi_totale: int,
#         montepremi_erogabile_totale: float,  # _type: €Int
#         jackpot_interno_al_gioco_erogabile: float,  # _type: €Int
#         jackpot_aggiuntivo_erogabile: float,  # _type: €Int
#         numero_premi_sessione_gioco: int,
#         numero_premio_iniziale: int,
#         numero_premio_finale: int,
#         montepremi_parziale: float,  # _type: €Int
#         jackpot_parziale_interno_al_gioco_erogabile: float,  # _type: €Int
#         jackpot_parziale_aggiuntivo_al_gioco_erogabile: float,  # _type: €Int
#         numero_premi_per_invio: int,
#         progressivo_rettifica: int,
#         lista_premi: list
#     ):
#         """
#         Questo messaggio consente ad un concessionario di comunicare il piano dei premi che
#         verranno distribuiti al termine del gioco e tutti i dati definitivi relativi al montepremi
#         qualora questi non fossero noti al momento dell’inizio della sessione di gioco.
#
#         Gli importi dei premi devono essere indicati in ORDINE DECRESCENTE!
#
#         I campi 6 (numero_premio_iniziale) e 7 (numero_premio_finale) devono essere impostati al
#         primo e all’ultimo numero dei premi presenti
#         nel messaggio inviato rispettando la sequenza dei premi e degli invii (restando
#         nell’esempio precedente in cui il campo 5 vale 1.500, saranno impostati a 1 e 1.000 nel
#         primo messaggio inviato, a 1.001 e 1.500 nel secondo messaggio inviato).
#
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param perc_montepremi_totale: Percentuale del montepremi erogabile totale. Nel caso in cui in un torneo la
#             percentuale del montepremi erogato totale supera il 100% della raccolta (per esempio se il
#             torneo prevede una somma minima garantita), deve essere impostato a 10000 (100%).
#         :param montepremi_erogabile_totale: Importo totale del montepremi erogabile derivante dalla raccolta
#             comprensivo di eventuali jackpot
#         :param jackpot_interno_al_gioco_erogabile: Quota del montepremi totale derivante dal jackpot interno
#             al gioco erogabile
#         :param jackpot_aggiuntivo_erogabile: Quota del montepremi totale derivante dal jackpot aggiuntivo erogabile
#         :param numero_premi_sessione_gioco: Numero totale dei premi da attribuire nella sessione di gioco.
#             Il campo `numero_premi_sessione_gioco` deve essere impostato al numero totale dei premi previsti
#             per la sessione di gioco; il numero n contenente il valore di campo 10 potrà al massimo essere pari a
#             1.000. Pertanto se il valore di campo 5 è maggiore di 1.000 il messaggio dovrà essere
#             inviato un numero di volte tali da completare la lista (ad esempio se campo 5 vale
#             1.500, si dovranno effettuare 2 invii: il primo contenente i primi 1.000 premi, con
#             campo 10 = 1.000, e il secondo gli altri 500 e campo 10= 500).
#         :param numero_premio_iniziale: Numero del premio iniziale relativo all’invio.
#         :param numero_premio_finale: Numero del premio finale relativo all’invio
#         :param montepremi_parziale: Importo del montepremi relativo ai premi presenti nell’invio
#             comprensivo di eventuali jackpot.
#             Contiene il valore della somma degli importi dei premi presenti nell’invio;
#             coinciderà con il campo 3 (montepremi erogabile) qualora la lista dei premi sia inferiore
#             a 1.000 e quindi il messaggio 240 sia inviato una sola volta.
#         :param jackpot_parziale_interno_al_gioco_erogabile: Quota del montepremi parziale derivante da jackpot
#             interno al gioco relativo ai premi presenti nell’invio.
#             Contiene il valore della somma degli importi dei premi, derivanti da Jackpot
#             interni al gioco, presenti nell’invio; coinciderà con il campo 4 (jackpot interno al gioco)
#             qualora la lista dei premi sia inferiore a 1.000 e quindi il messaggio 240 sia inviato una sola volta.
#         :param jackpot_parziale_aggiuntivo_al_gioco_erogabile: Quota del montepremi parziale derivante da
#             jackpot aggiuntivo relativo ai premi presenti nell’invio.
#             Contiene il valore della somma degli importi dei premi, derivanti da Jackpot
#             aggiuntivi, presenti nell’invio; coinciderà con il campo 4a (jackpot aggiuntivo) qualora
#             la lista dei premi sia inferiore a 1.000 e quindi il messaggio 240 sia inviato una sola volta.
#         :param numero_premi_per_invio: Contiene il numero (n) dei premi presenti nel messaggio (max 1000)
#         :param progressivo_rettifica: Vale 0 per il primo piano inviato, oppure il numero di volte in cui
#             è richiesta la rettifica (a partire dal valore 1 per la prima rettifica e incrementando di
#             1 per le successive).
#             nella maggior parte dei casi assume il valore 0 (piano premi iniziale),
#             mentre deve essere impostato con valori > 0 per ogni rettifica alla lista premi già
#             comunicata. La rettifica viene accettata dal sistema centrale solo se ancora non è stata
#             accettata la lista vincitori (messaggio 260).
#
#             La rettifica deve riguardare l’intero piano dei premi; pertanto, qualora siano da variare
#             solo alcuni dei dati registrati, deve essere effettuato comunque l’invio per l’intera lista
#             dei premi (es.: se è stata effettuata la comunicazione di un piano dei premi dal 1 al 20
#             (unico invio), e si ritengono errati i dati dal 5 al 10, nella rettifica devono essere indicati
#             tutti i premi previsti per la sessione, ordinati in maniera decrescente a partire da quello
#             relativo all’importo più elevato. Di conseguenza, se la comunicazione fosse avvenuta
#             con invii successivi, non può essere rettificato solo uno dei messaggi contenente una
#             parte dei premi, ma devono essere effettuati tanti invii per quanti premi previsti per la
#             sessione.
#         :param lista_premi: Lista dei premi nel seguente formato: [
#                 {"n": 1, "prize": 100, "jackpot_interno": 0, "jackpot_aggiuntivo": 0},
#                 ...
#             ]
#         """
#         return self._do_request_message(
#             message_code=240, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             perc_montepremi_totale=perc_montepremi_totale,
#             montepremi_erogabile_totale=montepremi_erogabile_totale,
#             jackpot_interno_al_gioco_erogabile=jackpot_interno_al_gioco_erogabile,
#             jackpot_aggiuntivo_erogabile=jackpot_aggiuntivo_erogabile,
#             numero_premi_sessione_gioco=numero_premi_sessione_gioco,
#             numero_premio_iniziale=numero_premio_iniziale,
#             numero_premio_finale=numero_premio_finale,
#             montepremi_parziale=montepremi_parziale,
#             jackpot_parziale_interno_al_gioco_erogabile=jackpot_parziale_interno_al_gioco_erogabile,
#             jackpot_parziale_aggiuntivo_al_gioco_erogabile=jackpot_parziale_aggiuntivo_al_gioco_erogabile,
#             numero_premi_per_invio=numero_premi_per_invio,
#             progressivo_rettifica=progressivo_rettifica,
#             lista_premi=lista_premi
#         )
#
#     def validate_and_prepare_data_240(
#         self,
#         id_sess_sist_centr: str,
#         perc_montepremi_totale: int,
#         montepremi_erogabile_totale: float,  # _type: €Int
#         jackpot_interno_al_gioco_erogabile: float,  # _type: €Int
#         jackpot_aggiuntivo_erogabile: float,  # _type: €Int
#         numero_premi_sessione_gioco: int,
#         numero_premio_iniziale: int,
#         numero_premio_finale: int,
#         montepremi_parziale: float,  # _type: €Int
#         jackpot_parziale_interno_al_gioco_erogabile: float,  # _type: €Int
#         jackpot_parziale_aggiuntivo_al_gioco_erogabile: float,  # _type: €Int
#         numero_premi_per_invio: int,
#         progressivo_rettifica: int,
#         lista_premi: list
#     ):
#         # VALIDATION
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         self.check_numeric_value('perc_montepremi_totale', perc_montepremi_totale, -1, "g")
#         self.check_numeric_value('perc_montepremi_totale', perc_montepremi_totale, 101, "l")
#         self.check_numeric_value('numero_premio_iniziale', numero_premio_iniziale, numero_premio_finale + 1, "l")
#
#         # ordina la lista premi
#         lista_premi = sorted(lista_premi, key=lambda x: x["n"])
#         # verifica che i premi siano in ordine decrescente
#
#         def monotonic_payout_structure(prizes):
#             """The prizes in `lista_premi` should satisfy monotonicity."""
#             if not all(x >= y for x, y in zip(prizes, prizes[1:])):
#                 raise ValidationError("The prizes should satisfy monotonicity.")
#
#         monotonic_payout_structure([prize['prize'] for prize in lista_premi])
#
#         try:
#             self.check_numeric_value('numero_premi_per_invio', numero_premi_per_invio, 1001, "l")
#         except ValidationError:
#             raise ValidationError("`numero_premi_per_invio` must be less than 1001. "
#                                   "Execute more than ona call as specified in function docstring "
#                                   "and documentation.")
#
#         self.check_numeric_value('len lista_premi', len(lista_premi), numero_premi_per_invio, "e")
#
#         # PREPARE DATA
#         self.data = {
#             # session
#             "campo1": id_sess_sist_centr,
#             "campo2": perc_montepremi_totale,
#             "campo3": montepremi_erogabile_totale,
#             "campo4": jackpot_interno_al_gioco_erogabile,
#             "campo4a": jackpot_aggiuntivo_erogabile,
#             "campo5": numero_premi_sessione_gioco,
#             "campo6": numero_premio_iniziale,
#             "campo7": numero_premio_finale,
#             "campo8": montepremi_parziale,
#             "campo9": jackpot_parziale_interno_al_gioco_erogabile,
#             "campo9a": jackpot_parziale_aggiuntivo_al_gioco_erogabile,
#             "campo10": numero_premi_per_invio,
#             "campo11": progressivo_rettifica,
#             "campo12": [
#                 {
#                     "campo12.1": d["prize"],
#                     "campo12.2": d["jackpot_interno"],
#                     "campo12.3": d["jackpot_aggiuntivo"],
#                 } for d in lista_premi
#             ],
#         }
#
#     # 250
#     def convalida_della_sessione_250(
#         self,
#         id_sess_sist_centr: str,
#         data_convalida: datetime,
#         id_sezione_in: str,
#
#     ):
#         """
#         Tramite questo messaggio il concessionario comunica il verificarsi dell’evento che
#         condiziona l’avvio del gioco nel flusso in modalità 2 (cfr. paragrafo 5.4.2).
#
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param data_convalida: Data convalida sessione di gioco
#         :param id_sezione_in: Lettera identificativa della sezione di primo livello di un torneo multifase
#              (lettera identificativa della sezione di ingresso). Da indicare per i tornei multifase,
#              valorizzandolo con una lettera progressiva tra A e la lettera corrispondente al valore massimo indicati
#              per l’attributo NSI del messaggio 200, cioè da A a massimo P. Indicare 0 se il torneo non è multifase
#              cioè non prevede livelli multipli con differenti sezioni di ingresso.
#         """
#         return self._do_request_message(
#             message_code=250, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             data_convalida=data_convalida,
#             id_sezione_in=id_sezione_in
#         )
#
#     def validate_and_prepare_data_250(
#         self,
#         id_sess_sist_centr: str,
#         data_convalida: datetime,
#         id_sezione_in: str,
#     ):
#         # VALIDATION
#         id_sezione_in = id_sezione_in.upper() if isinstance(id_sezione_in, str) else id_sezione_in
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         self.check_timezone('data_convalida', data_convalida, pytz.utc)
#         self.check_length('id_sezione_in', id_sezione_in, 1, "e")
#         self.check_value_in('id_sezione_in', id_sezione_in, "0ABCDEFGHILMNOP")
#
#         # PREPARE DATA
#         self.data = {
#             "campo1": id_sess_sist_centr,
#             "campo2": data_convalida.day,
#             "campo3": data_convalida.month,
#             "campo4": data_convalida.year,
#             "campo5": data_convalida.hour,
#             "campo6": data_convalida.minute,
#             "campo7": data_convalida.second,
#             "campo8": id_sezione_in,
#         }
#
#     # 260
#     def lista_vincitori_260(
#         self,
#         id_sess_sist_centr: str,
#         tot_vincitori_sessione: int,
#         progressivo_vincita_iniziale: int,
#         progressivo_vincita_finale: int,
#         imp_parziale_vincite: float,  # _type: €Int
#         imp_parziale_vincite_jackpot_interno: float,  # _type: €Int
#         imp_parziale_vincite_jackpot_aggiuntivo: float,  # _type: €Int
#         imp_parziale_vincite_bonus: float,  # _type: €Int
#         num_vincite: int,
#         progressivo_rettifica: int,
#         lista_vincite: list,
#         imp_tot_vincite: float,  # _type: €Int
#         imp_tot_vincite_jackpot_interno: float,  # _type: €Int
#         imp_tot_vincite_jackpot_aggiuntivo: float,  # _type: €Int
#         imp_tot_vincite_bonus: float,  # _type: €Int
#     ):
#         """
#         Questo messaggio consente ad un concessionario di comunicare le vincite realizzate in
#         una sessione di gioco.
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param tot_vincitori_sessione: Numero totale vincitori nella sessione di gioco. Deve essere
#             impostato con il numero totale dei vincitori della sessione; il
#             numero n contenente il valore di campo 6 (numero_vincite_invio) potrà al massimo essere pari a 1.000.
#             Pertanto se il campo 2 è maggiore di 1.000 il messaggio dovrà essere inviato un numero
#             di volte tali da completare la lista dei biglietti vincenti (ad esempio se campo 2 vale
#             1.500, si dovranno effettuare 2 invii: il primo contenente i primi 1.000 vincitori, campo
#             6 = 1.000, e il secondo gli altri 500, campo 6 = 500) ordinati in modo decrescente
#             rispetto al valore della vincita (somma dell’importo da montepremi e/o da jackpot).
#         :param progressivo_vincita_iniziale: Progressivo vincita iniziale dell’invio.
#         :param progressivo_vincita_finale: Progressivo vincita finale dell’invio.
#             progressivo_vincita_iniziale e progressivo_vincita_finale devono essere impostati
#             al primo e all’ultimo progressivo assegnato ai
#             vincitori presenti nel messaggio inviato, rispettando la sequenza delle vincite e degli
#             invii (restando nell’esempio precedente in cui il campo 2 vale 1.500, saranno impostati a
#             1 e 1.000 nel primo messaggio inviato, a 1.001 e 1.500 nel secondo messaggio inviato).
#         :param imp_parziale_vincite: Importo relativo alle vincite presenti nell’invio comprensivo di jackpot e di bonus.
#             Contiene il valore della somma degli importi delle vincite presenti nell’invio;
#             sarà pari all’importo totale dei premi erogati qualora il campo 2 (numero totale dei
#             vincitori) sia inferiore a 1.000 e quindi il messaggio 260 sia inviato una sola volta.
#         :param imp_parziale_vincite_jackpot_interno: Quota delle vincite presenti nell’invio
#             derivanti da jackpot interno al gioco
#         :param imp_parziale_vincite_jackpot_aggiuntivo: Quota delle vincite presenti nell’invio
#             derivanti da jackpot aggiuntivo
#         :param imp_parziale_vincite_bonus: Quota delle vincite bonus presenti nell’invio
#         :param num_vincite: Contiene il numero (n) delle vincite presenti nel messaggio
#         :param progressivo_rettifica: nella maggior parte dei casi assume il valore 0, (lista vincitori iniziale),
#             mentre deve essere impostato con valori > 0 per ogni rettifica alla lista premi già
#             comunicata. Il messaggio di rettifica dovrà essere utilizzato, nel caso di sessioni di gioco
#             CUP, nella comunicazione di più liste vincitori. In tal caso le liste vincitori successive
#             alla prima comunicata, dovranno contenere tutte le vincite verificatesi fino al momento
#             della trasmissione.
#         :param lista_vincite: Lista delle vincite nel seguente formato: [
#                 {
#                     "n": 1,
#                     "prize": 100,
#                     "jackpot_interno": 0,
#                     "jackpot_aggiuntivo": 0,
#                     "id_partecipazione": "584848494",
#                     "bonus": 0
#                 },
#                 ...
#             ]
#         :param imp_tot_vincite: Importo totale delle vincite erogate nella sessione di gioco
#             comprensivo di jackpot e di bonus
#         :param imp_tot_vincite_jackpot_interno: Quota delle vincite erogate nella sessione di gioco derivante dal
#             jackpot interno al gioco (vale 0 se non previsto)
#         :param imp_tot_vincite_jackpot_aggiuntivo: Quota delle vincite erogate nella sessione di gioco derivante dal
#             jackpot aggiuntivo (vale 0 se non previsto)
#         :param imp_tot_vincite_bonus: Quota delle vincite erogate nella sessione di gioco derivante dal
#             bonus (vale 0 se non previsto)
#         """
#         return self._do_request_message(
#             message_code=260, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             tot_vincitori_sessione=tot_vincitori_sessione,
#             progressivo_vincita_iniziale=progressivo_vincita_iniziale,
#             progressivo_vincita_finale=progressivo_vincita_finale,
#             imp_parziale_vincite=imp_parziale_vincite,
#             imp_parziale_vincite_jackpot_interno=imp_parziale_vincite_jackpot_interno,
#             imp_parziale_vincite_jackpot_aggiuntivo=imp_parziale_vincite_jackpot_aggiuntivo,
#             imp_parziale_vincite_bonus=imp_parziale_vincite_bonus,
#             num_vincite=num_vincite,
#             progressivo_rettifica=progressivo_rettifica,
#             lista_vincite=lista_vincite,
#             imp_tot_vincite=imp_tot_vincite,
#             imp_tot_vincite_jackpot_interno=imp_tot_vincite_jackpot_interno,
#             imp_tot_vincite_jackpot_aggiuntivo=imp_tot_vincite_jackpot_aggiuntivo,
#             imp_tot_vincite_bonus=imp_tot_vincite_bonus,
#         )
#
#     def validate_and_prepare_data_260(
#         self,
#         id_sess_sist_centr: str,
#         tot_vincitori_sessione: int,
#         progressivo_vincita_iniziale: int,
#         progressivo_vincita_finale: int,
#         imp_parziale_vincite: float,
#         imp_parziale_vincite_jackpot_interno: float,
#         imp_parziale_vincite_jackpot_aggiuntivo: float,
#         imp_parziale_vincite_bonus: float,
#         num_vincite: int,
#         progressivo_rettifica: int,
#         lista_vincite: list,
#         imp_tot_vincite: float,
#         imp_tot_vincite_jackpot_interno: float,
#         imp_tot_vincite_jackpot_aggiuntivo: float,
#         imp_tot_vincite_bonus: float,
#     ):
#         # VALIDATION
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         self.check_numeric_value('tot_vincitori_sessione', tot_vincitori_sessione, -1, "g")
#         self.check_numeric_value('len lista_vincite', num_vincite, len(lista_vincite), "e")
#         self.check_numeric_value('len lista_vincite', len(lista_vincite), tot_vincitori_sessione + 1, "l")
#         self.check_numeric_value(
#             'progressivo_vincita_iniziale', progressivo_vincita_iniziale, progressivo_vincita_finale + 1, "l"
#         )
#         self.check_numeric_value('imp_parziale_vincite', imp_parziale_vincite, -1, "g")
#         self.check_numeric_value(
#             'Somma lista premi jackpot interno',
#             imp_parziale_vincite_jackpot_interno,
#             sum([d["jackpot_interno"] for d in lista_vincite]),
#             "e"
#         )
#         self.check_numeric_value(
#             'Somma lista premi jackpot aggiuntivo',
#             imp_parziale_vincite_jackpot_aggiuntivo,
#             sum([d["jackpot_aggiuntivo"] for d in lista_vincite]),
#             "e"
#         )
#         self.check_numeric_value(
#             'Somma lista premi bonus',
#             imp_parziale_vincite_bonus,
#             sum([d["bonus"] for d in lista_vincite]),
#             "e"
#         )
#
#         # PREPARE DATA
#         self.data = {
#             "campo1": id_sess_sist_centr,
#             "campo2": tot_vincitori_sessione,
#             "campo3": progressivo_vincita_iniziale,
#             "campo4": progressivo_vincita_finale,
#             "campo5": imp_parziale_vincite,
#             "campo5a": imp_parziale_vincite_jackpot_interno,
#             "campo5b": imp_parziale_vincite_jackpot_aggiuntivo,
#             "campo5c": imp_parziale_vincite_bonus,
#             "campo6": num_vincite,
#             "campo7": progressivo_rettifica,
#             "campo8": [
#                 {
#                     "campo8.1": d["prize"],
#                     "campo8.2": d["jackpot_interno"],
#                     "campo8.3": d["id_partecipazione"],
#                     "campo8.4": d["jackpot_aggiuntivo"],
#                     "campo8.5": d["bonus"],
#                 } for d in sorted(lista_vincite, key=lambda x: x["n"])
#             ],
#             "campo9": imp_tot_vincite,
#             "campo10": imp_tot_vincite_jackpot_interno,
#             "campo11": imp_tot_vincite_jackpot_aggiuntivo,
#             "campo12": imp_tot_vincite_bonus,
#         }
#
#     # 280
#     def accredito_vincita_280(
#         self,
#         id_sess_sist_centr: str,
#         code_concessionario: int,
#         code_rete: int,
#         importo: float,  # _type: €Int
#         importo_bonus: float,  # _type: €Int
#         data_accredito: datetime,
#         code_conto_gioco: str,
#         progressivo_accredito: int = 1,
#     ):
#         """
#         Questo messaggio consente ad un concessionario di comunicare il pagamento di tutte le
#         vincite ottenute dal giocatore nella stessa sessione di gioco, tramite accredito della
#         somma sul conto di gioco del giocatore.
#
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param code_concessionario: Codice identificativo, assegnato da Aams, del concessionario/ titolare del
#             conto di gioco
#         :param code_rete: Codice identificativo della rete del concessionario titolare di sistema presso
#             cui è aperto il conto di gioco (vedi tabella § 8.6)
#         :param importo: Importo accreditato
#         :param importo_bonus: Quota dell’importo accreditato come “bonus”
#         :param data_accredito: Data dell'accredito vincita
#         :param code_conto_gioco: Numero del conto di gioco
#         :param progressivo_accredito: Vale 1 per il primo accredito inviato per il conto di gioco, oppure il numero di volte
#             in cui è richiesto l’accredito (incrementando di 1 per le successive comunicazioni).
#             Potrà avere un valore maggiore di 1 solo per gli accrediti delle vincite
#             parziali nel caso di sessioni di gioco CUP.
#         """
#         return self._do_request_message(
#             message_code=280, signed=True,
#             id_sess_sist_centr=id_sess_sist_centr,
#             code_concessionario=code_concessionario,
#             code_rete=code_rete,
#             importo=importo,
#             importo_bonus=importo_bonus,
#             data_accredito=data_accredito,
#             code_conto_gioco=code_conto_gioco,
#             progressivo_accredito=progressivo_accredito,
#         )
#
#     def validate_and_prepare_data_280(
#         self,
#         id_sess_sist_centr: str,
#         code_concessionario: int,
#         code_rete: int,
#         importo: float,
#         importo_bonus: float,
#         data_accredito: datetime,
#         code_conto_gioco: str,
#         progressivo_accredito: int,
#     ):
#         # VALIDATION
#         self.check_length('id_sess_sist_centr', id_sess_sist_centr, 16, "e")
#         self.check_enum('code_rete', code_rete, CodeRete)
#         self.check_numeric_value('importo', importo, 0, "g")
#         self.check_numeric_value('importo_bonus', importo_bonus, importo, "le")
#         self.check_timezone('data_accredito', data_accredito, pytz.utc)
#         self.check_length('code_conto_gioco', code_conto_gioco, 21, "l")
#         self.check_numeric_value('progressivo_accredito', progressivo_accredito, 1, "ge")
#
#         # PREPARE DATA
#         self.data = {
#             # identificativi sessione e concessionario
#             "campo1": id_sess_sist_centr,
#             "campo2": code_concessionario,
#             "campo3": code_rete,
#             # importi
#             "campo4": importo,
#             "campo4a": importo_bonus,
#             # data accredito
#             "campo5": data_accredito.day,
#             "campo6": data_accredito.month,
#             "campo7": data_accredito.year,
#             "campo8": data_accredito.hour,
#             "campo9": data_accredito.minute,
#             "campo10": data_accredito.second,
#             # conto gioco
#             "campo11": len(code_conto_gioco),
#             "campo12": code_conto_gioco,
#         }
#
#     # 300
#     def fine_sessione_300(self):
#         return self._do_request_message(
#             message_code=300, signed=True,
#             # TODO
#         )
#
#     # MESSAGGI PER LA GESTIONE DEGLI ARCHIVI - Comuni a tutte le tipologie di gioco/modalità di flusso
#     # 800
#     def richiesta_dati_rendicondazione_contabile_800(self):
#         return self._do_request_message(
#             message_code=800, signed=True,
#             # TODO
#         )
#
#     # 810
#     def richiesta_aggiornamento_data_fine_sessione_810(self):
#         return self._do_request_message(
#             message_code=810, signed=True,
#             # TODO
#         )
#
#     # 830
#     def comunicazione_del_software_installato_830(self):
#         return self._do_request_message(
#             message_code=830, signed=True,
#             # TODO
#         )
#
#     # 831
#     def comunicazione_del_software_installato_per_versione_831(self):
#         return self._do_request_message(
#             message_code=831, signed=True,
#             # TODO
#         )
#
#     # 840
#     def aggiunta_del_certificato_di_firma_840(self):
#         return self._do_request_message(
#             message_code=840, signed=True,
#             # TODO
#         )
#
#     # MESSAGGI PER LA GESTIONE DEGLI ARCHIVI - GIOCO IN MODALITA’ 1 E 2
#     # 310
#     def richiesta_invalidazione_sessione_310(self):
#         """
#         Tramite questo messaggio il concessionario comunica la richiesta di autorizzazione
#         all’invalidazione della sessione di gioco (solo nei casi esplicitamente previsti da AAMS).
#         """
#         return self._do_request_message(
#             message_code=310, signed=True,
#             # TODO
#         )
#
#     # 320
#     def richiesta_invalidazione_diritto_partecipazione_320(self):
#         """
#         """
#         return self._do_request_message(
#             message_code=320, signed=True,
#             # TODO
#         )
#
#     # 330
#     def verifica_stato_richiesta_invalidazione_330(self):
#         """
#         """
#         return self._do_request_message(
#             message_code=330, signed=True,
#             # TODO
#         )
#
#     def accredito_rimborso_340(self):
#         """
#         """
#         return self._do_request_message(
#             message_code=340, signed=True,
#             # TODO
#         )
#
#     def verifica_correttezza_sessioni_effettuate_360(self):
#         """
#         """
#         return self._do_request_message(
#             message_code=360, signed=True,
#             # TODO
#         )
#
#     def richiesta_notifica_anomalie_riscontrate_per_sessione_365(self):
#         """
#         """
#         return self._do_request_message(
#             message_code=365, signed=True,
#             # TODO
#         )
#
#     # RESPONSE HANDLING
#     def handle_generic_message_response(self, response: Response) -> tuple:
#         result = self.decode_response_body(response)
#
#         esito = result.get('Esito')  # or campo1 todo
#         if esito == 0:
#             return True, result
#
#         return False, self.get_aams_message_error(esito)
#
#     def handle_message_200_response(self, response: Response) -> str:
#         """Gestisce la risposta alla richiesta con messaggio 200.
#
#         Ritorna l'identificativo della sessione di gioco or raise AAMSException
#         """
#         success, msg_or_exception = self.handle_generic_message_response(response)
#         if success:
#             return msg_or_exception['campo2']  # todo verificare se campo 2 o altro nome
#         raise msg_or_exception
#
#     def handle_message_220_response(self, response: Response) -> tuple:
#         """Gestisce la risposta alla richiesta con messaggio 220.
#
#         Ritorna l'identificativo di partecipazione e la data di registrazione della
#         partecipazione sul sistema centrale di AAMS , quindi attribuita secondo la data in vigore
#         in Italia al momento dell’acquisizione del messaggio.
#         oppure raise AAMSException
#         """
#         success, msg_or_exception = self.handle_generic_message_response(response)
#         if success:
#             return (
#                 msg_or_exception['Identificativo di partecipazione'],
#                 datetime(year=msg_or_exception['Anno'],
#                          month=msg_or_exception['Mese'],
#                          day=msg_or_exception['Giorno'],
#                          tzinfo=pytz.utc)
#             )
#         raise msg_or_exception
#
#     def handle_message_230_response(self, response: Response) -> datetime:
#         """Gestisce la risposta alla richiesta con messaggio 230.
#
#         I campi da 2 a 4 del messaggio di risposta rappresentano la data di registrazione
#         dell’annullamento del diritto di partecipazione sul sistema centrale di AAMS, quindi
#         attribuita secondo la data in vigore in Italia al momento dell’acquisizione del messaggio.
#
#         Il messaggio non viene accettato dal sistema centrale se il concessionario ha già
#         trasmesso il messaggio di convalida della sessione (messaggio 250).
#
#         Analogamente, il messaggio che contiene la lista dei vincitori (messaggio 260) o quello
#         che contiene l’accredito di una vincita (messaggio 280), non può essere accettato se i
#         dati si riferiscono a identificativi di partecipazione in precedenza annullati.
#
#         oppure raise AAMSException
#         """
#         success, msg_or_exception = self.handle_generic_message_response(response)
#         if success:
#             return datetime(year=msg_or_exception['Anno'],
#                             month=msg_or_exception['Mese'],
#                             day=msg_or_exception['Giorno'],
#                             tzinfo=pytz.utc)
#         raise msg_or_exception
#
#     def handle_message_240_response(self, response: Response) -> None:
#         """Gestisce la risposta alla richiesta con messaggio 240.
#
#         :return: None o raise AAMSException
#         """
#         success, msg_or_exception = self.handle_generic_message_response(response)
#         if success:
#             return
#         raise msg_or_exception
#
#     def handle_message_250_response(self, response: Response) -> tuple:
#         """Gestisce la risposta alla richiesta con messaggio 250.
#
#         I campi da 2 a 4 del messaggio di risposta rappresentano la data di registrazione della
#         convalida sul sistema centrale di AAMS, quindi attribuita secondo la data in vigore in
#         Italia al momento dell’acquisizione del messaggio.
#
#         :return: tuple (Num_part_convalidate, Imp_part_convalidate, data) or raise AAMSException
#         """
#         success, msg_or_exception = self.handle_generic_message_response(response)
#         if success:
#             return (
#                 msg_or_exception["Num_part_convalidate"],
#                 msg_or_exception["Imp_part_convalidate"],
#                 datetime(year=msg_or_exception['Anno'],
#                          month=msg_or_exception['Mese'],
#                          day=msg_or_exception['Giorno'],
#                          tzinfo=pytz.utc)
#             )
#         raise msg_or_exception
