# # -*- coding: utf-8 -*-
# import json
# import logging
# import uuid
# from datetime import datetime
# from typing import Optional
# 
# from thx.giocaonline.exceptions import ValidationError, AAMSException
# 
# from thx.giocaonline import AAMSClient
# 
# 
# # todo set correct logger
# logger = logging.getLogger("exceptions")
# 
# 
# class AAMSHandler:
# 
#     DEFAULT_COMMUNICATION_MODALITY = "2"
# 
#     def __init__(self, client_klass=AAMSClient, communication_modality: Optional[str] = None, *args, **kwargs):
#         self.client: AAMSClient = client_klass(*args, **kwargs)
#         self.communication_modality = communication_modality or self.DEFAULT_COMMUNICATION_MODALITY
# 
#     def show_available_methods(self):
#         self.client.show_available_methods()
# 
#     def inizio_sessione_200(
#         self,
#         session_datetime_start: datetime,
#         session_datetime_end: datetime,
#         imp_diritto_partecip: float,  # _type: €Int
#         # session attributes
#         tipo_montepremi: str,
#         perc_montepremi: int,
#         rby: int,
#         num_gioc_previsti: int,
#         somma_min_garantita: Optional[int] = None,
#         bonus: bool = False,
#         vin: bool = False,
#         jk1: bool = False,
#         jk2: bool = False,
#         cup: bool = False,
#         nsi: Optional[int] = None,
#         # optional params
#         session_id: Optional[str] = None,
#         communication_modality: Optional[str] = None,
#     ) -> tuple:
#         """
#         Gestisce la prima comunicazione con AAMS per l'inizio di una nuova sessione di gioco.
# 
#         :param session_datetime_start: Data inizio sessione di gioco (UTC)
#         :param session_datetime_end: Data di fine sessione di gioco (UTC).
#             Deve essere impostata alla data fine della sessione di gioco “presunta”
#             (si intende valida fino alle ore 24 del giorno indicato); qualora il
#             concessionario si accorga, durante lo svolgimento del gioco, di non riuscire ad effettuare
#             la chiusura entro la data dichiarata, dovrà inoltrare il messaggio di proroga (messaggio
#             810), al fine di poter completare correttamente la sessione di gioco; I limiti della durata
#             di una sessione di gioco sono determinati da ADM.
#         :param imp_diritto_partecip: Importo del diritto di partecipazione
#         :param tipo_montepremi:
#             P = "percentuale"
#             S = "somma minima garantita"
#             V = "montepremi variabile"
#         :param perc_montepremi: Indica la percentuale montepremi. A partire da 8000. Per TPM=V indicare la percentuale media.
#         :param rby: Definisce la sessione in cui è consentito il riacquisto del biglietto.
#                 0 = "nessuno"
#                 1 = "rebuy"
#                 2 = "add-on"
#                 3 = "entrambi"
#         :param bonus: True se il bonus è ammesso, altrimenti False. Default False.
#         :param vin: Deve essere impostato qualora il concessionario apre la sessione di gioco
#             prevedendo l’utilizzo delle vincite in essa ottenute, per l’acquisto del diritto di
#             partecipazione in una sessione di gioco futura.
#         :param jk1: deve essere impostato True qualora il concessionario apre la sessione di gioco
#             che prevede erogazione di jackpot, derivanti dalla matematica del gioco, che
#             incrementano la vincita spettante al giocatore.
#         :param jk2: deve essere impostato True qualora il concessionario apre la sessione di gioco
#             che prevede erogazione di jackpot aggiuntivi, assegnati indipendentemente dal gioco e
#             che si aggiungono alla percentuale di RTP per la determinazione del margine per il concessionario.
#         :param cup: bool, default False. Indica se la sessione di gioco è in modalità CUP
#         :param nsi: Numero delle sezioni del primo livello di un torneo multifase (da 1 a 16)
#         :param num_gioc_previsti: Numero dei giocatori previsti
#         :param somma_min_garantita: somma minima garantita (obbligatorio se tipo_montepremi = "S")
#         :param session_id: Optional, se non passato viene generato un identificativo univoco.
#         :param communication_modality: Flusso di gestione delle comunicazioni prescelto dal concessionario (vale 1 o 2).
#             Default = "1"
#         :return: tuple, (session_id, id_sess_sist_centr)
#             session_id: id della sezione indicato dall'utente o calcolato dal metodo
#             id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema di convalida (in caso di esito positivo)
#             oppure raise Exception (AAMSException o ValidationError)
#         """
#         logger.info("****************************")
#         logger.info("Start `inizio_sessione_200`.")
# 
#         try:
#             if not session_id:
#                 session_id = str(uuid.uuid4())
# 
#             if not communication_modality:
#                 communication_modality = self.communication_modality
# 
#             if tipo_montepremi == "S" and not somma_min_garantita:
#                 e = ValidationError("`somma_min_garantita` cannot be empty if tipo_montepremi = `S`")
#                 logger.error(e)
#                 logger.info("End `inizio_sessione_200` with errors.")
#                 logger.info("****************************")
#                 raise e
# 
#             logger.info("session_id: %s" % session_id)
#             logger.info("communication_modality: %s" % communication_modality)
#             logger.info("session_datetime_start: %s" % session_datetime_start.strftime("%Y/%m/%d %H:%M:%S"))
#             logger.info("session_datetime_end: %s" % session_datetime_end.strftime("%Y/%m/%d %H:%M:%S"))
#             logger.info("imp_diritto_partecip: %d" % imp_diritto_partecip)
#             logger.info("tipo_montepremi: %s" % tipo_montepremi)
#             logger.info("perc_montepremi: %d" % perc_montepremi)
#             logger.info("somma_min_garantita: %d" % somma_min_garantita)
#             logger.info("rby: %d" % rby)
#             logger.info("num_gioc_previsti: %d" % num_gioc_previsti)
#             logger.info("bonus: %s" % bonus)
#             logger.info("vin: %s" % vin)
#             logger.info("jk1: %s" % jk1)
#             logger.info("jk2: %s" % jk2)
#             logger.info("cup: %s" % cup)
#             logger.info("nsi: %d" % nsi)
# 
#             # prepare session attributes
#             session_attributes = dict(
#                 TPM=tipo_montepremi,
#                 PRM=perc_montepremi,
#                 RBY=rby,
#                 NGM=num_gioc_previsti,
#                 JK1=int(jk1),
#                 JK2=int(jk2),
#                 CUP=int(cup),
#             )
#             if somma_min_garantita:
#                 session_attributes["SMG"] = somma_min_garantita
#             if bonus is True:
#                 session_attributes["BON"] = "B"
#             if vin is True:
#                 session_attributes["VIN"] = "V"
#             if nsi:
#                 session_attributes["NSI"] = nsi
# 
#             try:
#                 id_sess_sist_centr = self.client.inizio_sessione_200(
#                     session_id=session_id,
#                     session_datetime_start=session_datetime_start,
#                     session_datetime_end=session_datetime_end,
#                     communication_modality=str(communication_modality),
#                     imp_diritto_partecip=imp_diritto_partecip,
#                     **session_attributes
#                 )
#             except ValidationError as e:
#                 self.handle_validation_error(e)
#                 raise e
#             except AAMSException as e:
#                 self.handle_aams_error(e)
#                 raise e
#             except Exception as e:
#                 self.handle_generic_error(e)
#                 raise e
# 
#         except Exception as e:
#             logger.info("End `inizio_sessione_200` with errors.")
#             logger.info("****************************")
#             raise e
# 
#         logger.info("End `inizio_sessione_200` successfully.")
#         logger.info("****************************")
#         return session_id, id_sess_sist_centr
# 
#     def acquisto_diritto_di_partecipazione_220(
#         self,
#         id_sess_sist_centr: str,
#         data: dict,
#         id_sezione_in: str = "0",
#     ) -> tuple:
#         """
#         Gestisce l'invio ad AAMS degli acquisti dei diritti di partecipazione degli utenti per UN SOLO TORNEO
#         (un unico identificativo di sessione).
# 
#         Esegue l'operazione per tutte le giocate, distinguendo le giocate che terminano con successo da quelle che
#         falliscono.
# 
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale con
#             il messaggio 200
#         :param id_sezione_in: Lettera identificativa della sezione di primo livello di un torneo multifase
#              (lettera identificativa della sezione di ingresso). Da indicare per i tornei multifase,
#              valorizzandolo con una lettera progressiva tra A e la lettera corrispondente al valore massimo indicati
#              per l’attributo NSI del messaggio 200, cioè da A a massimo P. Indicare 0 se il torneo non è multifase.
#         :param data: Oggetto contenente i dati relativi al singolo utente. Deve essere nel seguente formato:
#             {
#                 "cod_regione": "01",
#                 "imp_partecipazione": 10.00,
#                 "imp_partecipazione_bonus": 0,
#                 "imp_a_fondo_jackpot": 0,
#                 "flag_riacquisto": 0,
#                 "ip_address": "127.0.0.1",
#                 "code_concessionario": 1234,
#                 "code_rete": 2,
#                 "code_conto_gioco": "dnsubfyevfefbv",
#                 "pseudonimo_giocatore": "Giocatore 1",
#                 "data": datetime.datetime(2021, 5, 24, 12, 59, 7, 537001),
#                 "code_tipo_tag": "10",
#                 # opzionali:
#                 "id_sessione_collegata": "515748sd848s5cs7",
#             },
# 
#         :return: tuple (Identificativo di partecipazione, datetime) or raise Exception (AAMSException o ValidationError)
#         """
#         logger.info("****************************")
#         logger.info("Start `acquisto_diritto_di_partecipazione_220`.")
#         try:
#             attributes = [
#                 "cod_regione",
#                 "imp_partecipazione",
#                 "imp_partecipazione_bonus",
#                 "imp_a_fondo_jackpot",
#                 "flag_riacquisto",
#                 "ip_address",
#                 "code_concessionario",
#                 "code_rete",
#                 "code_conto_gioco",
#                 "pseudonimo_giocatore",
#             ]
# 
#             # valida che tutti gli elementi abbiano gli attributi richiesti
#             try:
#                 for attr in attributes:
#                     data[attr]  # noqa
#             except AttributeError as e:
#                 raise ValidationError("`data`: invalid `data` format. Missing key `%s`." % str(e))
# 
#             logger.info("id_sess_sist_centr: %s" % id_sess_sist_centr)
#             logger.info("id_sezione_in: %s" % id_sezione_in)
#             logger.info("data: %s" % str(data))
# 
#             try:
#                 result = self.client.acquisto_diritto_di_partecipazione_220(
#                     id_sess_sist_centr=id_sess_sist_centr,
#                     id_sezione_in=id_sezione_in,
#                     **data
#                 )
#             except ValidationError as e:
#                 self.handle_validation_error(e)
#                 raise e
#             except AAMSException as e:
#                 self.handle_aams_error(e)
#                 raise e
#             except Exception as e:
#                 self.handle_generic_error(e)
#                 raise e
# 
#         except Exception as e:
#             logger.info("End `acquisto_diritto_di_partecipazione_220` with errors.")
#             logger.info("****************************")
#             raise e
# 
#         logger.info("End `acquisto_diritto_di_partecipazione_220` successfully.")
#         logger.info("****************************")
# 
#         return result
# 
#     def acquisto_diritto_di_partecipazione_bulk_220(
#         self,
#         id_sess_sist_centr: str,
#         data: list,
#         id_sezione_in: str = "0",
#     ):
#         """
#         Gestisce l'invio ad AAMS degli acquisti dei diritti di partecipazione degli utenti per UN SOLO TORNEO
#         (un unico identificativo di sessione).
# 
#         Esegue l'operazione per tutte le giocate, distinguendo le giocate che terminano con successo da quelle che
#         falliscono.
# 
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale con
#             il messaggio 200
#         :param id_sezione_in: Lettera identificativa della sezione di primo livello di un torneo multifase
#              (lettera identificativa della sezione di ingresso). Da indicare per i tornei multifase,
#              valorizzandolo con una lettera progressiva tra A e la lettera corrispondente al valore massimo indicati
#              per l’attributo NSI del messaggio 200, cioè da A a massimo P. Indicare 0 se il torneo non è multifase.
#         :param data: Lista di oggetti contenenti i dati relativi ai singoli utenti. Devono essere nel seguente formato:
#             [
#                 {
#                     "n": 0,
#                     "cod_regione": "01",
#                     "imp_partecipazione": 10.00,
#                     "imp_partecipazione_bonus": 0,
#                     "imp_a_fondo_jackpot": 0,
#                     "flag_riacquisto": 0,
#                     "ip_address": "127.0.0.1",
#                     "code_concessionario": 1234,
#                     "code_rete": 2,
#                     "code_conto_gioco": "dnsubfyevfefbv",
#                     "pseudonimo_giocatore": "Giocatore 1",
#                     "data": datetime.datetime(2021, 5, 24, 12, 59, 7, 537001),
#                     "code_tipo_tag": "10",
#                     # opzionali:
#                     "id_sessione_collegata": "515748sd848scs7",
#                 },
#                 ...
#             ]
# 
#         :return: tuple (n, (Identificativo di partecipazione, datetime) or Exception, row)
#         """
#         logger.info("****************************")
#         logger.info("Start `acquisto_diritto_di_partecipazione_bulk_220`.")
#         try:
#             attributes = [
#                 "n",
#                 "cod_regione",
#                 "imp_partecipazione",
#                 "imp_partecipazione_bonus",
#                 "imp_a_fondo_jackpot",
#                 "flag_riacquisto",
#                 "ip_address",
#                 "code_concessionario",
#                 "code_rete",
#                 "code_conto_gioco",
#                 "pseudonimo_giocatore",
#             ]
# 
#             # valida che tutti gli elementi abbiano gli attributi richiesti
#             for row, cnt in enumerate(data):
#                 try:
#                     for attr in attributes:
#                         row[attr]  # noqa
#                 except AttributeError as e:
#                     raise ValidationError("`data`: invalid format for element %d. Missing key `%s`." % (cnt, str(e)))
# 
#             logger.info("id_sess_sist_centr: %s" % id_sess_sist_centr)
#             logger.info("id_sezione_in: %s" % id_sezione_in)
# 
#             results = []
#             for row in data:
#                 n = row.pop("n")
# 
#                 logger.info("row: %s" % json.dumps(row))
# 
#                 try:
#                     result = self.client.acquisto_diritto_di_partecipazione_220(
#                         id_sess_sist_centr=id_sess_sist_centr,
#                         id_sezione_in=id_sezione_in,
#                         **row
#                     )
#                     results.append((n, result, row))
#                 except ValidationError as e:
#                     self.handle_validation_error(e)
#                     results.append((n, e, row))
#                 except AAMSException as e:
#                     self.handle_aams_error(e)
#                     results.append((n, e, row))
#                 except Exception as e:
#                     self.handle_generic_error(e)
#                     results.append((n, e, row))
# 
#         except Exception as e:
#             logger.info("End `acquisto_diritto_di_partecipazione_bulk_220` with errors.")
#             logger.info("****************************")
#             raise e
# 
#         logger.info("End `acquisto_diritto_di_partecipazione_bulk_220` successfully.")
#         logger.info("****************************")
# 
#         return results
# 
#     def richiesta_annullamento_diritto_di_partecipazione_230(
#         self,
#         id_sess_sist_centr: str,
#         id_partecipazione: str,
#     ):
#         """
#         Il concessionario può richiedere l’annullamento di un diritto
#         di partecipazione acquistato in precedenza, solo per sessione di gioco in modalità 2 e
#         prima che sia stata convalidata (cfr paragrafo 5.4.2).
#         In altri casi l’annullamento può avvenire per motivi esclusivamente tecnici e disciplinati
#         da AAMS in appositi provvedimenti amministrativi, e tramite il flusso appositamente
#         predisposto (cfr paragrafo 5.5).
# 
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param id_partecipazione: Codice univoco del diritto di partecipazione attribuito dal sistema centrale
#         :return: datetime, la data di annullamento del diritto di partecipazione o raise (AAMSException o ValidationError)
#         """
#         logger.info("****************************")
#         logger.info("Start `richiesta_annullamento_diritto_di_partecipazione_230`.")
#         try:
# 
#             logger.info("id_sess_sist_centr: %s" % id_sess_sist_centr)
#             logger.info("id_partecipazione: %s" % id_partecipazione)
# 
#             try:
#                 data = self.client.richiesta_annullamento_diritto_di_partecipazione_230(
#                     id_sess_sist_centr=id_sess_sist_centr,
#                     id_partecipazione=id_partecipazione,
#                 )
#             except ValidationError as e:
#                 self.handle_validation_error(e)
#                 raise e
#             except AAMSException as e:
#                 self.handle_aams_error(e)
#                 raise e
#             except Exception as e:
#                 self.handle_generic_error(e)
#                 raise e
# 
#         except Exception as e:
#             logger.info("End `richiesta_annullamento_diritto_di_partecipazione_230` with errors.")
#             logger.info("****************************")
#             raise e
# 
#         logger.info("End `richiesta_annullamento_diritto_di_partecipazione_230` successfully.")
#         logger.info("****************************")
#         return data
# 
#     def piano_dei_premi_240(
#         self,
#         id_sess_sist_centr: str,
#         perc_montepremi_totale: int,
#         montepremi_erogabile_totale: float,  # _type: €Int
#         jackpot_interno_al_gioco_erogabile: float,  # _type: €Int
#         jackpot_aggiuntivo_erogabile: float,  # _type: €Int
#         numero_premi_sessione_gioco: int,
#         numero_premio_iniziale: int,
#         lista_premi: list,
#     ):
#         """
# 
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param perc_montepremi_totale: Percentuale del montepremi erogabile totale. Nel caso in cui in un torneo la
#             percentuale del montepremi erogato totale supera il 100% della raccolta (per esempio se il
#             torneo prevede una somma minima garantita), deve essere impostato a 10000 (100%).
#         :param montepremi_erogabile_totale: Importo totale del montepremi erogabile derivante dalla raccolta
#             comprensivo di eventuali jackpot
#         :param jackpot_interno_al_gioco_erogabile: Quota del montepremi totale derivante dal jackpot interno
#             al gioco erogabile
#         :param jackpot_aggiuntivo_erogabile: Quota del montepremi totale derivante dal jackpot aggiuntivo erogabile
#         :param numero_premi_sessione_gioco: Numero totale dei premi da attribuire nella sessione di gioco.
#             Il campo `numero_premi_sessione_gioco` deve essere impostato al numero totale dei premi previsti
#             per la sessione di gioco; il numero n contenente il valore di campo 10 potrà al massimo essere pari a
#             1.000. Pertanto se il valore di campo 5 è maggiore di 1.000 il messaggio dovrà essere
#             inviato un numero di volte tali da completare la lista (ad esempio se campo 5 vale
#             1.500, si dovranno effettuare 2 invii: il primo contenente i primi 1.000 premi, con
#             campo 10 = 1.000, e il secondo gli altri 500 e campo 10= 500).
#         :param lista_premi: Lista dei premi nel seguente formato: [
#             {
#                 "n": 1,
#                 "prize": 100,
#                 "jackpot_interno": 0,
#                 "jackpot_aggiuntivo": 0,
#             },
#             ...
#         ]
#         """
#         logger.info("****************************")
#         logger.info("Start `piano_dei_premi_240`.")
# 
#         try:
#             result = self.client.piano_dei_premi_240(
#                 id_sess_sist_centr=id_sess_sist_centr,
#                 perc_montepremi_totale=perc_montepremi_totale,
#                 lista_premi=lista_premi
#             )
#         except Exception as e:
#             logger.info("End `piano_dei_premi_240` with errors.")
#             logger.info("****************************")
#             raise e
# 
#         logger.info("End `piano_dei_premi_240` successfully.")
#         logger.info("****************************")
# 
#     def lista_vincitori_260(
#         self,
#         id_sess_sist_centr: str,
#         lista_vincite: list,
#     ):
#         """
#         Gestisce l'invio ad AAMS della lista dei vincitori. Qualora la lista sia superiore a 1000 vincite,
#         il metodo gestisce autonomamente l'invio separato delle informazioni.
# 
#         :param id_sess_sist_centr: Codice identificativo della sessione attribuito dal sistema centrale
#         :param lista_vincite: Lista delle vincite nel seguente formato: [
#             {
#                 "n": 1,
#                 "prize": 100,
#                 "jackpot_interno": 0,
#                 "jackpot_aggiuntivo": 0,
#                 "id_partecipazione": "584848494",
#                 "bonus": 0
#             },
#             ...
#         ]
#         """
#         # TODO gestire il risultato
#         progressivo_rettifica = 0
#         progressivo_vincita_iniziale = 1
#         tot_vincitori_sessione = len(lista_vincite)
# 
#         def calcola_importi(_lista_vincite: list):
#             tot, jackpot_interno, jackpot_aggiuntivo, bonus = 0, 0, 0, 0
#             for win in _lista_vincite:
#                 tot += win["prize"]
#                 jackpot_interno += win["jackpot_interno"]
#                 jackpot_aggiuntivo += win["jackpot_aggiuntivo"]
#                 bonus += win["bonus"]
# 
#             return {
#                 "tot": tot,
#                 "jackpot_interno": jackpot_interno,
#                 "jackpot_aggiuntivo": jackpot_aggiuntivo,
#                 "bonus": bonus,
#             }
# 
#         importi_totali = calcola_importi(lista_vincite)
# 
#         for lv in self.chunks(lista_vincite, 1000):
#             progressivo_vincita_finale = progressivo_vincita_iniziale + len(lv) - 1
#             importi_parziali = calcola_importi(lv)
# 
#             result = self.client.lista_vincitori_260(
#                 id_sess_sist_centr=id_sess_sist_centr,
#                 tot_vincitori_sessione=tot_vincitori_sessione,
#                 progressivo_vincita_iniziale=progressivo_vincita_iniziale,
#                 progressivo_vincita_finale=progressivo_vincita_finale,
#                 progressivo_rettifica=progressivo_rettifica,
#                 num_vincite=len(lv),
#                 lista_vincite=lv,
#                 # importi parziali
#                 imp_parziale_vincite=importi_parziali["tot"],
#                 imp_parziale_vincite_jackpot_interno=importi_parziali["jackpot_interno"],
#                 imp_parziale_vincite_jackpot_aggiuntivo=importi_parziali["jackpot_aggiuntivo"],
#                 imp_parziale_vincite_bonus=importi_parziali["bonus"],
#                 # importi totali
#                 imp_tot_vincite=importi_totali["tot"],
#                 imp_tot_vincite_jackpot_interno=importi_totali["jackpot_interno"],
#                 imp_tot_vincite_jackpot_aggiuntivo=importi_totali["jackpot_aggiuntivo"],
#                 imp_tot_vincite_bonus=importi_totali["bonus"],
#             )
# 
#             progressivo_rettifica += 1
#             progressivo_vincita_iniziale += progressivo_vincita_finale
# 
#     def handle_validation_error(self, validation_error, *args, **kwargs):
#         """Hook this method to handle Validation Errors in a different way."""
#         self.handle_generic_error(validation_error, *args, **kwargs)
# 
#     def handle_aams_error(self, aams_error, *args, **kwargs):
#         """Hook this method to handle AAMS Errors in a different way."""
#         self.handle_generic_error(aams_error, *args, **kwargs)
# 
#     def handle_generic_error(self, error, *args, **kwargs):  # noqa
#         """Hook this method to handle Generic Errors in a different way."""
#         logger.error(error)
# 
#     @staticmethod
#     def chunks(lst: list, n: int):
#         """Yield successive n-sized chunks from lst."""
#         for i in range(0, len(lst), n):
#             yield lst[i:i + n]
